@@ -1,4 +1,5 @@
 """Phonopy command user interface."""
+
 # Copyright (C) 2020 Atsushi Togo
 # All rights reserved.
 #
@@ -60,6 +61,7 @@ from phonopy.file_IO import (
     write_FORCE_CONSTANTS,
     write_force_constants_to_hdf5,
 )
+from phonopy.harmonic.dynamical_matrix import DynamicalMatrixNAC
 from phonopy.harmonic.force_constants import (
     compact_fc_to_full_fc,
     full_fc_to_compact_fc,
@@ -284,13 +286,13 @@ def print_settings(
     interface_mode = phonon.calculator
     run_mode = settings.run_mode
     if interface_mode:
-        print("Calculator interface: %s" % interface_mode)
+        print(f"Calculator interface: {interface_mode}")
+    print(f'Crystal structure was read from "{unitcell_filename}".')
     if (
         settings.cell_filename is not None
         and settings.cell_filename != unitcell_filename
-    ):  # noqa E129
-        print('"%s" was not able to be used.' % settings.cell_filename)
-    print('Crystal structure was read from "%s".' % unitcell_filename)
+    ):
+        print(f'("{settings.cell_filename}" was not used though specified.)')
     physical_units = get_default_physical_units(interface_mode)
     print("Unit of length: %s" % physical_units["length_unit"])
     if is_band_auto(settings) and not is_primitive_axes_auto:
@@ -423,7 +425,7 @@ def print_settings(
 
 
 def write_displacements_files_then_exit(
-    phonon, settings, confs, optional_structure_info, log_level
+    phonon: Phonopy, settings, confs, optional_structure_info, log_level
 ):
     """Write supercells with displacements and displacement dataset.
 
@@ -439,7 +441,7 @@ def write_displacements_files_then_exit(
         phonon.calculator,
         phonon.supercell,
         cells_with_disps,
-        optional_structure_info,
+        optional_structure_info=optional_structure_info,
         additional_info=additional_info,
     )
 
@@ -852,7 +854,7 @@ def store_nac_params(
             if log_level:
                 dm = phonon.dynamical_matrix
                 if dm is not None:
-                    if dm.is_nac():
+                    if isinstance(dm, DynamicalMatrixNAC):
                         dm.show_nac_message()
                     print("")
 
@@ -872,7 +874,7 @@ def store_nac_params(
                 print("-" * 76)
 
 
-def run(phonon: Phonopy, settings, plot_conf, log_level):
+def run_calculation(phonon: Phonopy, settings, plot_conf, log_level):
     """Run phonon calculations."""
     interface_mode = phonon.calculator
     physical_units = get_default_physical_units(interface_mode)
@@ -1056,7 +1058,6 @@ def run(phonon: Phonopy, settings, plot_conf, log_level):
         # Thermal property
         #
         if settings.is_thermal_properties:
-
             if log_level:
                 if settings.is_projected_thermal_properties:
                     print("Calculating projected thermal properties...")
@@ -1425,13 +1426,14 @@ def run(phonon: Phonopy, settings, plot_conf, log_level):
             print_error()
             sys.exit(1)
 
-        phonon.set_modulations(
+        phonon.run_modulations(
             dimension,
             phonon_modes,
             delta_q=delta_q,
             derivative_order=derivative_order,
             nac_q_direction=settings.nac_q_direction,
         )
+
         phonon.write_modulations()
         phonon.write_yaml_modulations()
 
@@ -1490,19 +1492,25 @@ def start_phonopy(**argparse_control):
         if deprecated:
             show_deprecated_option_warnings(deprecated)
 
-    return (
-        args,
-        log_level,
-        {
-            "plot_graph": args.is_graph_plot,
-            "save_graph": args.is_graph_save,
-            "with_legend": args.is_legend,
-        },
-    )
+    return args, log_level
 
 
 def read_phonopy_settings(args, argparse_control, log_level):
-    """Read phonopy settings."""
+    """Read phonopy settings.
+
+    Returns
+    -------
+    tuple
+        settings : PhonopySettings
+            Configurations set by user to run phonopy.
+        confs : dict
+            Raw phonopy configurations in str (value) for each configuration
+            tag (key).
+        cell_filename : str or None
+            Filename that contains crystal structure information. When
+            unspecified in command line tool, this is None.
+
+    """
     load_phonopy_yaml = argparse_control.get("load_phonopy_yaml", False)
     conf_filename = None
 
@@ -1525,6 +1533,7 @@ def read_phonopy_settings(args, argparse_control, log_level):
             cell_filename = phonopy_conf_parser.settings.cell_filename
     else:
         if len(args.filename) > 0:
+            file_exists(args.filename[0], log_level)
             if is_file_phonopy_yaml(args.filename[0]):
                 phonopy_conf_parser = PhonopyConfParser(args=args)
                 cell_filename = args.filename[0]
@@ -1542,7 +1551,7 @@ def read_phonopy_settings(args, argparse_control, log_level):
     settings = phonopy_conf_parser.settings
 
     if log_level > 0 and conf_filename is not None:
-        print('Phonopy configuration was read from "%s".' % conf_filename)
+        print(f'"{conf_filename}" was read as phonopy configuration file.')
 
     return settings, confs, cell_filename
 
@@ -1744,7 +1753,17 @@ def main(**argparse_control):
     ############################################
     load_phonopy_yaml = argparse_control.get("load_phonopy_yaml", False)
 
-    args, log_level, plot_conf = start_phonopy(**argparse_control)
+    if "args" in argparse_control:  # For pytest
+        args = argparse_control["args"]
+        log_level = args.log_level
+    else:
+        args, log_level = start_phonopy(**argparse_control)
+
+    plot_conf = {
+        "plot_graph": args.is_graph_plot,
+        "save_graph": args.is_graph_save,
+        "with_legend": args.is_legend,
+    }
 
     settings, confs, cell_filename = read_phonopy_settings(
         args, argparse_control, log_level
@@ -1972,7 +1991,7 @@ def main(**argparse_control):
             print(" - %s" % mode)
         print("-" * 76)
 
-    run(phonon, settings, plot_conf, log_level)
+    run_calculation(phonon, settings, plot_conf, log_level)
 
     ########################
     # Phonopy finalization #
