@@ -1,4 +1,5 @@
 """File I/O related routines."""
+
 # Copyright (C) 2011 Atsushi Togo
 # All rights reserved.
 #
@@ -831,7 +832,8 @@ def is_file_phonopy_yaml(filename, keyword="phonopy"):
         ...
 
     """
-    with open(filename, "r") as f:
+    myio = get_io_module_to_decompress(filename)
+    with myio.open(filename, "r") as f:
         try:
             data = yaml.load(f, Loader=Loader)
             if data is None:
