@@ -1,4 +1,5 @@
 """Routines to handle various calculator interfaces."""
+
 # Copyright (C) 2014 Atsushi Togo
 # All rights reserved.
 #
@@ -32,6 +33,7 @@
 # LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
+from __future__ import annotations
 
 import os
 from argparse import ArgumentParser
@@ -66,17 +68,18 @@ calculator_info = {
     "abacus": {"option": {"name": "--abacus", "help": "Invoke ABACUS mode"}},
     "abinit": {"option": {"name": "--abinit", "help": "Invoke Abinit mode"}},
     "aims": {"option": {"name": "--aims", "help": "Invoke FHI-aims mode"}},
+    "castep": {"option": {"name": "--castep", "help": "Invoke CASTEP mode"}},
     "cp2k": {"option": {"name": "--cp2k", "help": "Invoke CP2K mode"}},
     "crystal": {"option": {"name": "--crystal", "help": "Invoke CRYSTAL mode"}},
     "dftbp": {"option": {"name": "--dftb+", "help": "Invoke dftb+ mode"}},
     "elk": {"option": {"name": "--elk", "help": "Invoke elk mode"}},
+    "fleur": {"option": {"name": "--fleur", "help": "Invoke Fleur mode"}},
+    "lammps": {"option": {"name": "--lammps", "help": "Invoke Lammps mode"}},
     "qe": {"option": {"name": "--qe", "help": "Invoke Quantum espresso (QE) mode"}},
     "siesta": {"option": {"name": "--siesta", "help": "Invoke Siesta mode"}},
     "turbomole": {"option": {"name": "--turbomole", "help": "Invoke TURBOMOLE mode"}},
     "vasp": {"option": {"name": "--vasp", "help": "Invoke Vasp mode"}},
     "wien2k": {"option": {"name": "--wien2k", "help": "Invoke Wien2k mode"}},
-    "castep": {"option": {"name": "--castep", "help": "Invoke CASTEP mode"}},
-    "fleur": {"option": {"name": "--fleur", "help": "Invoke Fleur mode"}},
 }
 
 
@@ -199,7 +202,12 @@ def write_crystal_structure(
 
         pps = optional_structure_info[1]
         orbitals = optional_structure_info[2]
-        abacus.write_abacus(filename, cell, pps, orbitals)
+        abfs = optional_structure_info[3]
+        abacus.write_abacus(filename, cell, pps, orbitals, abfs)
+    elif interface_mode == "lammps":
+        import phonopy.interface.lammps as lammps
+
+        lammps.write_lammps(filename, cell)
     else:
         raise RuntimeError("No calculator interface was found.")
 
@@ -208,7 +216,7 @@ def write_supercells_with_displacements(
     interface_mode,
     supercell,
     cells_with_disps,
-    optional_structure_info,
+    optional_structure_info=None,
     displacement_ids=None,
     zfill_width=3,
     additional_info=None,
@@ -243,7 +251,7 @@ def write_supercells_with_displacements(
 
     args = (supercell, cells_with_disps, ids)
     kwargs = {"width": zfill_width}
-    if "pre_filename" in additional_info:
+    if additional_info is not None and "pre_filename" in additional_info:
         kwargs["pre_filename"] = additional_info["pre_filename"]
 
     if interface_mode is None or interface_mode == "vasp":
@@ -328,8 +336,13 @@ def write_supercells_with_displacements(
 
         pps = optional_structure_info[1]
         orbitals = optional_structure_info[2]
-        abacus_args = args + (pps, orbitals)
+        abfs = optional_structure_info[3]
+        abacus_args = args + (pps, orbitals, abfs)
         abacus.write_supercells_with_displacements(*abacus_args, **kwargs)
+    elif interface_mode == "lammps":
+        import phonopy.interface.lammps as lammps
+
+        lammps.write_supercells_with_displacements(*args, **kwargs)
     else:
         raise RuntimeError("No calculator interface was found.")
 
@@ -354,7 +367,10 @@ def write_magnetic_moments(cell, sort_by_elements=False):
 
 
 def read_crystal_structure(
-    filename=None, interface_mode=None, chemical_symbols=None, phonopy_yaml_cls=None
+    filename=None,
+    interface_mode=None,
+    chemical_symbols=None,
+    phonopy_yaml_cls: type[PhonopyYaml] = PhonopyYaml,
 ):
     """Return crystal structure from file in each calculator format.
 
@@ -385,10 +401,7 @@ def read_crystal_structure(
 
     """
     if interface_mode == "phonopy_yaml":
-        if phonopy_yaml_cls is None:
-            return _read_phonopy_yaml(filename, PhonopyYaml)
-        else:
-            return _read_phonopy_yaml(filename, phonopy_yaml_cls)
+        return _read_phonopy_yaml(filename, phonopy_yaml_cls)
 
     if filename is None:
         cell_filename = get_default_cell_filename(interface_mode)
@@ -470,8 +483,13 @@ def read_crystal_structure(
     elif interface_mode == "abacus":
         from phonopy.interface.abacus import read_abacus
 
-        unitcell, pps, orbitals = read_abacus(cell_filename, elements=chemical_symbols)
-        return unitcell, (cell_filename, pps, orbitals)
+        unitcell, pps, orbitals, abfs = read_abacus(cell_filename)
+        return unitcell, (cell_filename, pps, orbitals, abfs)
+    elif interface_mode == "lammps":
+        from phonopy.interface.lammps import read_lammps
+
+        unitcell = read_lammps(cell_filename)
+        return unitcell, (cell_filename,)
     else:
         raise RuntimeError("No calculator interface was found.")
 
@@ -498,12 +516,14 @@ def get_default_cell_filename(interface_mode):
         return "control"
     elif interface_mode == "aims":
         return "geometry.in"
-    elif interface_mode in ("castep"):
+    elif interface_mode == "castep":
         return "unitcell.cell"
     elif interface_mode == "fleur":
         return "fleur.in"
     elif interface_mode == "abacus":
         return "STRU"
+    elif interface_mode == "lammps":
+        return "unitcell"
     else:
         return None
 
@@ -531,10 +551,12 @@ def get_default_supercell_filename(interface_mode):
         return None  # TURBOMOLE interface generates directories with inputs
     elif interface_mode == "aims":
         return "geometry.in.supercell"
-    elif interface_mode in ("castep"):
+    elif interface_mode == "castep":
         return "supercell.cell"
-    elif interface_mode in ("abacus"):
+    elif interface_mode == "abacus":
         return "sSTRU"
+    elif interface_mode == "lammps":
+        return "supercell"
     else:
         return None
 
@@ -566,8 +588,8 @@ def get_default_physical_units(interface_mode=None):
     abinit        : hartree, au,        AMU,         eV/angstrom,  eV/angstrom.au
     elk           : hartree, au,        AMU,         hartree/au,   hartree/au^2
     qe            : Ry,      au,        AMU,         Ry/au,        Ry/au^2
-    siesta        : eV,      au,        AMU,         eV/Angstroem, eV/angstrom.au
-    CRYSTAL       : eV,      angstrom,  AMU,         eV/Angstroem, eV/angstrom^2
+    siesta        : eV,      au,        AMU,         eV/angstrom,  eV/angstrom.au
+    CRYSTAL       : eV,      angstrom,  AMU,         eV/angstrom,  eV/angstrom^2
     DFTB+         : hartree, au,        AMU          hartree/au,   hartree/au^2
     TURBOMOLE     : hartree, au,        AMU,         hartree/au,   hartree/au^2
     CP2K          : hartree, angstrom,  AMU,         hartree/au,   hartree/angstrom.au
@@ -575,6 +597,7 @@ def get_default_physical_units(interface_mode=None):
     castep        : eV,      angstrom,  AMU,         eV/angstrom,  eV/angstrom^2
     fleur         : hartree, au,        AMU,         hartree/au,   hartree/au^2
     abacus        : eV,      au,        AMU,         eV/angstrom,  eV/angstrom.au
+    lammps        : eV,      angstrom,  AMU,         eV/angstrom,  eV/angstrom^2
 
     units['force_constants_unit'] is used in
     the 'get_force_constant_conversion_factor' method.
@@ -587,20 +610,23 @@ def get_default_physical_units(interface_mode=None):
         "force_to_eVperA": None,
         "force_constants_unit": None,
         "length_unit": None,
+        "force_unit": None,
     }
 
-    if interface_mode is None or interface_mode in ("vasp", "aims"):
+    if interface_mode is None or interface_mode in ("vasp", "aims", "lammps"):
         units["factor"] = VaspToTHz
         units["nac_factor"] = Hartree * Bohr
         units["distance_to_A"] = 1.0
         units["force_constants_unit"] = "eV/angstrom^2"
         units["length_unit"] = "angstrom"
+        units["force_unit"] = "eV/angstrom"
     elif interface_mode == "abinit":
         units["factor"] = AbinitToTHz
         units["nac_factor"] = Hartree / Bohr
         units["distance_to_A"] = Bohr
         units["force_constants_unit"] = "eV/angstrom.au"
         units["length_unit"] = "au"
+        units["force_unit"] = "eV/angstrom"
     elif interface_mode == "qe":
         units["factor"] = PwscfToTHz
         units["nac_factor"] = 2.0
@@ -608,42 +634,49 @@ def get_default_physical_units(interface_mode=None):
         units["force_to_eVperA"] = Rydberg / Bohr
         units["force_constants_unit"] = "Ry/au^2"
         units["length_unit"] = "au"
+        units["force_unit"] = "Ry/au"
     elif interface_mode == "wien2k":
         units["factor"] = Wien2kToTHz
         units["nac_factor"] = 2000.0
         units["distance_to_A"] = Bohr
         units["force_constants_unit"] = "mRy/au^2"
         units["length_unit"] = "au"
+        units["force_unit"] = "mRy/au"
     elif interface_mode == "elk":
         units["factor"] = ElkToTHz
         units["nac_factor"] = 1.0
         units["distance_to_A"] = Bohr
         units["force_constants_unit"] = "hartree/au^2"
         units["length_unit"] = "au"
+        units["force_unit"] = "hartree/au"
     elif interface_mode in ["siesta", "abacus"]:
         units["factor"] = SiestaToTHz
         units["nac_factor"] = Hartree / Bohr
         units["distance_to_A"] = Bohr
         units["force_constants_unit"] = "eV/angstrom.au"
         units["length_unit"] = "au"
+        units["force_unit"] = "eV/angstrom"
     elif interface_mode == "cp2k":
         units["factor"] = CP2KToTHz
         units["nac_factor"] = None  # not implemented
         units["distance_to_A"] = 1.0
         units["force_constants_unit"] = "hartree/angstrom.au"
         units["length_unit"] = "angstrom"
+        units["force_unit"] = "hartree/au"
     elif interface_mode == "crystal":
         units["factor"] = CrystalToTHz
         units["nac_factor"] = Hartree * Bohr
         units["distance_to_A"] = 1.0
         units["force_constants_unit"] = "eV/angstrom^2"
         units["length_unit"] = "angstrom"
+        units["force_unit"] = "eV/angstrom"
     elif interface_mode == "dftbp":
         units["factor"] = DftbpToTHz
         units["nac_factor"] = Hartree * Bohr
         units["distance_to_A"] = Bohr
         units["force_constants_unit"] = "hartree/au^2"
         units["length_unit"] = "au"
+        units["force_unit"] = "hartree/au"
     elif interface_mode == "turbomole":
         units["factor"] = TurbomoleToTHz
         units["nac_factor"] = 1.0
@@ -651,18 +684,21 @@ def get_default_physical_units(interface_mode=None):
         units["force_to_eVperA"] = Hartree / Bohr
         units["force_constants_unit"] = "hartree/au^2"
         units["length_unit"] = "au"
+        units["force_unit"] = "hartree/au"
     elif interface_mode == "castep":
         units["factor"] = CastepToTHz
         units["nac_factor"] = Hartree * Bohr
         units["distance_to_A"] = 1.0
         units["force_constants_unit"] = "eV/angstrom^2"
         units["length_unit"] = "angstrom"
+        units["force_unit"] = "eV/angstrom"
     elif interface_mode == "fleur":
         units["factor"] = FleurToTHz
         units["nac_factor"] = 1.0
         units["distance_to_A"] = Bohr
         units["force_constants_unit"] = "hartree/au^2"
         units["length_unit"] = "au"
+        units["force_unit"] = "hartree/au"
 
     return units
 
@@ -706,6 +742,8 @@ def get_force_sets(
         from phonopy.interface.fleur import parse_set_of_forces
     elif interface_mode == "abacus":
         from phonopy.interface.abacus import parse_set_of_forces
+    elif interface_mode == "lammps":
+        from phonopy.interface.lammps import parse_set_of_forces
 
     else:
         return []
@@ -761,7 +799,7 @@ def get_force_constant_conversion_factor(unit, interface_mode):
         return 1.0
 
 
-def _read_phonopy_yaml(filename, phonopy_yaml_cls):
+def _read_phonopy_yaml(filename, phonopy_yaml_cls: type[PhonopyYaml]):
     cell_filename = _get_cell_filename(filename, phonopy_yaml_cls)
     if cell_filename is None:
         return None, (None, None)
@@ -778,7 +816,7 @@ def _read_phonopy_yaml(filename, phonopy_yaml_cls):
     return cell, (cell_filename, phpy)
 
 
-def _get_cell_filename(filename, phonopy_yaml_cls):
+def _get_cell_filename(filename, phonopy_yaml_cls: type[PhonopyYaml]):
     cell_filename = None
 
     default_filenames = []
