@@ -1,4 +1,5 @@
 """VASP calculator interface."""
+
 # Copyright (C) 2011 Atsushi Togo
 # All rights reserved.
 #
@@ -33,13 +34,17 @@
 # ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 # POSSIBILITY OF SUCH DAMAGE.
 
+from __future__ import annotations
+
+import io
 import os
 import sys
 import warnings
 import xml.etree.cElementTree as etree
 import xml.parsers.expat
-from io import StringIO
-from typing import Optional
+from collections import Counter, abc
+from collections.abc import Sequence
+from typing import Optional, Union
 
 import numpy as np
 
@@ -90,20 +95,49 @@ def get_scaled_positions_lines(scaled_positions):
     return "\n".join(_get_scaled_positions_lines(scaled_positions))
 
 
-def sort_positions_by_symbols(symbols, positions):
-    """Sort atomic positions by symbols."""
-    from collections import Counter
+def sort_positions_by_symbols(symbols: abc.Sequence, positions: np.ndarray):
+    """Sort atomic positions by symbols.
 
-    # unique symbols in order of first appearance in 'symbols'
-    reduced_symbols = _unique_stable(symbols)
+    Sort positions by symbols (using the order defined by reduced_symbols)
+    using a stable sort algorithm. Written by @ExpHP, refactored by @atztogo.
 
-    # counts of each symbol
+    symbols = ["A", "B", "A", "B"]
+    reduced_symbols = ["A", "B"]
+    sort_keys = [0, 1, 0, 1]
+    perm = [0, 2, 1, 3]
+    counts_dict = {'A': 2, 'B': 2}
+    counts_list = [2, 2]
+
+    Parameters
+    ----------
+    symbols : list[str] or list[int] or np.ndarray[int]
+        Sequence of hashable objects. This may be a list of chemical symbols
+        or numbers.
+    positions : np.ndarray
+        Atomic positions.
+
+    Returns
+    -------
+    sorted_positions = positions[perm]
+    For the others, see the example above.
+
+    Functions
+    ---------
+    _argsort_stable :
+        Alternative to `np.argsort(keys)` that uses a stable sorting algorithm
+        so that indices tied for the same value are listed in increasing order.
+
+    """
+
+    def _argsort_stable(keys):
+        # Python's built-in sort algorithm is a stable sort
+        return sorted(range(len(keys)), key=keys.__getitem__)
+
+    # dict in Python 3.7 or later is ordered dict.
+    reduced_symbols = list(dict.fromkeys(symbols))
     counts_dict = Counter(symbols)
+    # list(counts_dict.values()) may be used...
     counts_list = [counts_dict[s] for s in reduced_symbols]
-
-    # sort positions by symbol (using the order defined by reduced_symbols).
-    # using a stable sort algorithm matches the behavior of previous versions
-    #  of phonopy (but is not otherwise necessary)
     sort_keys = [reduced_symbols.index(i) for i in symbols]
     perm = _argsort_stable(sort_keys)
     sorted_positions = positions[perm]
@@ -215,7 +249,7 @@ def read_vasp(filename, symbols=None):
 
 def read_vasp_from_strings(strings, symbols=None):
     """Parse POSCAR type string."""
-    return _get_atoms_from_poscar(StringIO(strings).readlines(), symbols)
+    return _get_atoms_from_poscar(io.StringIO(strings).readlines(), symbols)
 
 
 def _get_atoms_from_poscar(lines, symbols):
@@ -315,9 +349,37 @@ def write_vasp(filename, cell, direct=True):
         w.write("\n".join(lines))
 
 
-def get_vasp_structure_lines(cell, direct=True, is_vasp5=True):
-    """Generate POSCAR text lines as a list from PhonopyAtoms instance."""
-    lines, scaled_positions = _get_vasp_structure_header_lines(cell, is_vasp5=is_vasp5)
+def get_vasp_structure_lines(
+    cell: PhonopyAtoms,
+    direct: bool = True,
+    is_vasp5: bool = True,
+    is_vasp4: bool = False,
+    first_line_str: Optional[str] = None,
+):
+    """Generate POSCAR text lines as a list from PhonopyAtoms instance.
+
+    direct : bool
+        Dummy argument. This doesn nothing.
+    is_vasp5 : bool
+        Deprecated. This is replaced by ``is_vasp4 = not is_vasp5``.
+
+    """
+    _is_vasp4 = is_vasp4
+    if is_vasp5 is False:
+        warnings.warn(
+            "is_vasp5 parameter is deprecated. "
+            "Use is_vasp4=True instead of is_vasp5=False",
+            DeprecationWarning,
+        )
+        _is_vasp4 = True
+    if direct is False:
+        warnings.warn(
+            "direct=False is not supported. ",
+            DeprecationWarning,
+        )
+    lines, scaled_positions = _get_vasp_structure_header_lines(
+        cell, is_vasp4=_is_vasp4, first_line_str=first_line_str
+    )
     lines.append("Direct")
     lines += _get_scaled_positions_lines(scaled_positions)
 
@@ -340,19 +402,23 @@ def write_supercells_with_displacements(
         write_vasp(filename, cell, direct=True)
 
 
-def _get_vasp_structure_header_lines(cell: PhonopyAtoms, is_vasp5=True):
-    (num_atoms, symbols, scaled_positions, sort_list) = sort_positions_by_symbols(
+def _get_vasp_structure_header_lines(
+    cell: PhonopyAtoms, is_vasp4: bool = False, first_line_str: Optional[str] = None
+):
+    (num_atoms, symbols, scaled_positions, _) = sort_positions_by_symbols(
         cell.symbols, cell.scaled_positions
     )
     lines = []
-    if is_vasp5:
+    if is_vasp4:
+        lines.append(" ".join(["%s" % s for s in symbols]))
+    elif first_line_str is None:
         lines.append("generated by phonopy")
     else:
-        lines.append(" ".join(["%s" % s for s in symbols]))
+        lines.append(first_line_str)
     lines.append("   1.0")
     for a in cell.cell:
         lines.append("  %21.16f %21.16f %21.16f" % tuple(a))
-    if is_vasp5:
+    if not is_vasp4:
         lines.append(" ".join(["%s" % s for s in symbols]))
     lines.append(" ".join(["%4d" % n for n in num_atoms]))
     return lines, scaled_positions
@@ -370,26 +436,6 @@ def _get_scaled_positions_lines(scaled_positions):
     ]
 
 
-# Get all unique values from a iterable.
-# Unlike `list(set(iterable))`, this is a stable algorithm;
-# items are returned in order of their first appearance.
-def _unique_stable(iterable):
-    seen_list = []
-    seen_set = set()
-    for x in iterable:
-        if x not in seen_set:
-            seen_set.add(x)
-            seen_list.append(x)
-    return seen_list
-
-
-# Alternative to `np.argsort(keys)` that uses a stable sorting algorithm
-# so that indices tied for the same value are listed in increasing order
-def _argsort_stable(keys):
-    # Python's built-in sort algorithm is a stable sort
-    return sorted(range(len(keys)), key=keys.__getitem__)
-
-
 #
 # Non-analytical term
 #
@@ -585,16 +631,9 @@ class Vasprun:
 
         """
         fc_tmp = None
-        version_nums = None
+        hessian_units = ""
         num_atom = 0
         for event, element in vasprun_etree:
-            # VASP version
-            if element.tag == "generator":
-                for element_i in element.findall("./i"):
-                    if element_i.attrib["name"] == "version":
-                        version_str = element_i.text.strip()
-                        version_nums = version_str.split("-")[0].split(".")
-
             if num_atom == 0:
                 atomtypes = self._get_atomtypes(element)
                 if atomtypes:
@@ -604,17 +643,27 @@ class Vasprun:
                     for n, m in zip(num_atoms, elem_masses):
                         masses += [m] * n
 
-            # Get Hessian matrix (normalized by masses)
-            if element.tag == "varray":
-                if element.attrib["name"] == "hessian":
-                    fc_tmp = []
-                    for v in element.findall("./v"):
-                        fc_tmp.append([float(x) for x in v.text.strip().split()])
+            # Get dynmat node
+            if element.tag == "dynmat":
+                # Get Hessian matrix (normalized by masses)
+                v_elements = element.findall("./varray[@name='hessian']/v")
+                if v_elements is not None:
+                    fc_tmp = np.array(
+                        [[float(x) for x in v.text.strip().split()] for v in v_elements]
+                    )
+
+                # Get physical units of Hessian
+                unit_element = element.find("./i[@name='unit']")
+                if unit_element is not None:
+                    hessian_units = unit_element.text.strip()
+
+            # Stop parsing when we have all the information
+            if num_atom > 0 and fc_tmp is not None:
+                break
 
         if fc_tmp is None:
             return None, None
         else:
-            fc_tmp = np.array(fc_tmp)
             if fc_tmp.shape != (num_atom * 3, num_atom * 3):
                 return False
 
@@ -631,9 +680,8 @@ class Vasprun:
                     force_constants[i, j] *= -np.sqrt(masses[i] * masses[j])
 
             # Recover the unit of eV/Angstrom^2 for VASP-6.
-            if version_nums is not None and len(version_nums) > 1:
-                if int(version_nums[0]) == 6 and int(version_nums[1]) > 1:
-                    force_constants /= VaspToTHz**2
+            if hessian_units == "THz^2":
+                force_constants /= VaspToTHz**2
 
             return force_constants, elements
 
@@ -644,18 +692,15 @@ class Vasprun:
         num_atoms = []
 
         if element.tag == "atominfo":
-            for element_array in element.findall("./array"):
-                if (
-                    "name" in element_array.attrib
-                    and element_array.attrib["name"] == "atomtypes"
-                ):
-                    for rc in element_array.findall("./set/rc"):
-                        atom_info = [x.text for x in rc.findall("./c")]
-                        num_atoms.append(int(atom_info[0]))
-                        atom_types.append(atom_info[1].strip())
-                        masses.append(float(atom_info[2]))
-                        valences.append(float(atom_info[3]))
-                    return num_atoms, atom_types, masses, valences
+            rc_elements = element.findall("./array[@name='atomtypes']/set/rc")
+            if rc_elements is not None:
+                for rc in rc_elements:
+                    atom_info = [x.text for x in rc.findall("./c")]
+                    num_atoms.append(int(atom_info[0]))
+                    atom_types.append(atom_info[1].strip())
+                    masses.append(float(atom_info[2]))
+                    valences.append(float(atom_info[3]))
+                return num_atoms, atom_types, masses, valences
 
         return None
 
@@ -1560,14 +1605,12 @@ def _read_XDATCAR_fileptr(f):
     return lattice, numbers_of_atoms
 
 
-def write_XDATCAR(
+def get_XDATCAR_lines_from_vasprunxml(
     vasprunxml_filename: str = "vasprun.xml",
     vasprunxml_expat: Optional[VasprunxmlExpat] = None,
-    filename: str = "XDATCAR",
-    fileptr=None,
-    shift=None,
+    shift: Union[Sequence, np.ndarray] = None,
 ):
-    """Write XDATCAR from vasprun.xml or VasprunxmlExpat instance.
+    """Return XDATCAR lines from vasprun.xml or VasprunxmlExpat instance.
 
     vasprunxml_filename : str, optional
         File name of vasprun.xml. Default is "vasprun.xml". This is used unless
@@ -1598,7 +1641,26 @@ def write_XDATCAR(
     for i, scaled_positions in enumerate(points):
         lines.append(f"Direct configuration=    {i}")
         lines += _get_scaled_positions_lines(scaled_positions)
+    return lines
 
+
+def write_XDATCAR(
+    vasprunxml_filename: str = "vasprun.xml",
+    vasprunxml_expat: Optional[VasprunxmlExpat] = None,
+    filename: str = "XDATCAR",
+    fileptr=Union[str, bytes, os.PathLike, io.IOBase],
+    shift: Union[Sequence, np.ndarray] = None,
+):
+    """Write XDATCAR from vasprun.xml or VasprunxmlExpat instance.
+
+    See get_XDATCAR_lines_from_vasprunxml.
+
+    """
+    lines = get_XDATCAR_lines_from_vasprunxml(
+        vasprunxml_filename=vasprunxml_filename,
+        vasprunxml_expat=vasprunxml_expat,
+        shift=shift,
+    )
     if fileptr is None:
         with open(filename, "w") as w:
             w.write("\n".join(lines))
