@@ -1,4 +1,8 @@
 """Pytest configuration."""
+
+from __future__ import annotations
+
+from collections.abc import Callable
 from pathlib import Path
 from typing import Tuple
 
@@ -258,14 +262,14 @@ def convcell_cr() -> PhonopyAtoms:
 
 
 @pytest.fixture(scope="session")
-def helper_methods():
+def helper_methods() -> Callable:
     """Return methods to compare cells."""
 
     class HelperMethods:
         @classmethod
         def compare_cells_with_order(
             cls, cell: PhonopyAtoms, cell_ref: PhonopyAtoms, symprec=1e-5
-        ):
+        ) -> None:
             """Compare two cells with the same orders of positions."""
             np.testing.assert_allclose(cell.cell, cell_ref.cell, atol=symprec)
             cls.compare_positions_with_order(
@@ -281,7 +285,9 @@ def helper_methods():
                 )
 
         @classmethod
-        def compare_positions_with_order(cls, pos, pos_ref, lattice, symprec=1e-5):
+        def compare_positions_with_order(
+            cls, pos, pos_ref, lattice, symprec=1e-5
+        ) -> None:
             """Compare two lists of positions and orders.
 
             lattice :
@@ -296,7 +302,7 @@ def helper_methods():
         @classmethod
         def compare_cells(
             cls, cell: PhonopyAtoms, cell_ref: PhonopyAtoms, symprec=1e-5
-        ):
+        ) -> None:
             """Compare two cells where position orders can be different."""
             np.testing.assert_allclose(cell.cell, cell_ref.cell, atol=symprec)
 
@@ -319,7 +325,7 @@ def helper_methods():
         @classmethod
         def compare_positions_in_arbitrary_order(
             cls, pos_in, pos_ref, lattice, symprec=1e-5
-        ):
+        ) -> list:
             """Compare two sets of positions irrespective of orders.
 
             lattice :
